@@ -240,6 +240,7 @@ static int set_duid(struct connman_service *service,
 
 		hex_duid = convert_to_hex(duid, duid_len);
 		if (!hex_duid) {
+			g_free(duid);
 			g_key_file_free(keyfile);
 			return -ENOMEM;
 		}
@@ -505,7 +506,7 @@ static int set_other_addresses(GDHCPClient *dhcp_client,
 			for (i = 0, list = option; list;
 						list = list->next, i++)
 				domains[i] = g_strdup(list->data);
-			__connman_service_update_search_domains(service, domains);
+			__connman_service_set_search_domains(service, domains);
 			g_strfreev(domains);
 		}
 	}
