@@ -618,8 +618,8 @@ static gboolean technology_pending_reply(gpointer user_data)
 static int technology_affect_devices(struct connman_technology *technology,
 						bool enable_device)
 {
+	int err = 0, err_dev;
 	GSList *list;
-	int err = -ENXIO;
 
 	if (technology->type == CONNMAN_SERVICE_TYPE_P2P) {
 		if (enable_device)
@@ -633,9 +633,12 @@ static int technology_affect_devices(struct connman_technology *technology,
 		struct connman_device *device = list->data;
 
 		if (enable_device)
-			err = __connman_device_enable(device);
+			err_dev = __connman_device_enable(device);
 		else
-			err = __connman_device_disable(device);
+			err_dev = __connman_device_disable(device);
+
+		if (err_dev < 0 && err_dev != -EALREADY)
+			err = err_dev;
 	}
 
 	return err;
@@ -1142,6 +1145,13 @@ static void technology_put(struct connman_technology *technology)
 
 	g_slist_free(technology->device_list);
 
+    if (technology->pending_reply) {
+        dbus_message_unref(technology->pending_reply);
+        technology->pending_reply = NULL;
+        g_source_remove(technology->pending_timeout);
+        technology->pending_timeout = 0;
+    }
+
 	g_free(technology->path);
 	g_free(technology->regdom);
 	g_free(technology->tethering_ident);
@@ -1811,6 +1821,12 @@ void __connman_technology_cleanup(void)
 {
 	DBG("");
 
+	while (technology_list) {
+		struct connman_technology *technology = technology_list->data;
+		technology_list = g_slist_remove(technology_list, technology);
+		technology_put(technology);
+	}
+
 	g_hash_table_destroy(rfkill_list);
 
 	dbus_connection_unref(connection);
